%
% Gauss-Seidel-Verfahren
%------------------------------
%
%  Verfahren:
%    Iterationsmatrix: 
%                H = I-(D+L)^(-1)*A
%    Iterationsvorschrift: 
%          x_(k+1) = xk + (D+L)^(-1)(b-A*xk)
%
% Inputvariablen
%  - A    Zu untersuchende Matrix
%  - b    Lsg von Ax=b (Spaltenvektor)
%  - x0   Startvektor  (Spaltenvektor)
%  - imax Anzahl maximaler Iterationsschritte
%  - tol  Genauigkeit, bei deren Erreichen die Iteration abbricht
%         (Durch vergleich der relativen Residuen-Norm)
%
% Outputvariablen
%  - xk         Die letzte Iterierte
%
% Programmaufruf mit:
%
%   xk = jacobi(A,b,x0,imax,tol)
%
%   Die Parameter x0,imax und tol sind optional, falls nicht angegeben
%   werden die folgenden Standardwerte genutzt:
%
%     x0   = Nullvektor
%     imax = 1000
%     tol  = 1e-15
%
%
function  xk = gaussseidel(A,b,x0,imax,tol)

%% Falls nicht angegeben, Standardwerte festlegen
if (nargin < 2)
  error('gaussseidel:NotEnoughInputs', 'Not enough input arguments.');
end
if (nargin < 5)
  tol = 1e-15;
  if (nargin < 4)
    imax = 1000;
    if (nargin < 3)
      x0 = zeros(size(A,1),1);
    end
  end
end
xk = x0;

%% Initialisieren
%dim = size(A,1);
iter = 0;
res = b-A*xk;
resnorm = norm(res);
bnorm = norm(b);
DL = tril(A);

%% Iteration
while (iter < imax) && (resnorm/bnorm>tol)
    % neue Iterierte
    xk=xk+DL\res;
    % neues Residuum
    res = b-A*xk;
    resnorm = norm(res);
    iter = iter+1;
    if mod(iter,50)==0
       display(sprintf('iter=%d',iter));
    end
end